#!/usr/bin/env python
# -*- coding: utf-8 -*-

# People Presence Clint -- Client for Predesys People Presence service plugin
# Version: 0.1.0
# Author: Jose Antonio Jimenez Carmona <josantjim@gmail.com>
# License: LGPL 3.0 (see "license" file)
import sys
sys.dont_write_bytecode = True
import os
import os.path
import exceptions
import simplexml

# Configuration file path to load
CONFIG_FILE = "configuration.xml"

# Default settings
DEFAULT_TITLE = "People"
DEFAULT_PREDESYS_WEBSERVICE_ADDRESS = "http://localhost:8000"

class Configuration():
    def __init__(self):
        """Class constructor.
        
        It initialize the object with the default values.
        """
        self.set_default_values()
        
    def set_default_values(self):
        """Set the default values to all fields."""
        self.set_title(DEFAULT_TITLE)
        self.set_predesys_webservice_address(DEFAULT_PREDESYS_WEBSERVICE_ADDRESS)
             
    def get_title(self):
        return self.__title
        
    def get_predesys_webservice_address(self):
        return self.__predesysWebserviceAddress
        
    def set_title(self, title):
        if title != None and type(title) != str: raise coreexceptions.ValueNotValidException("'title' must be a string.")
        self.__title = title
        
    def set_predesys_webservice_address(self, address):
        if address != None and type(address) != str: raise coreexceptions.ValueNotValidException("'address' must be a string.")
        self.__predesysWebserviceAddress = address
    
    def load_from_file(self, path):
        """Loads settings from a XML file.
        
        Keyword arguments:
        path -- File path from wich load settings (string)
        
        It raises an exception (ValueNotValidException) if path isn't a valid file path.
        It raises an exception (FileNotFoundException) if file doesn't exist.
        It raises an exception (LoadFileException) if an error occurs loading the file.
        """
        if path == None or type(path) != str or path == "": raise coreexceptions.ValueNotValidException("Path must be a valid file path.")
        if not (os.path.exists(path) and os.path.isfile(path)): raise coreexceptions.FileNotFoundException("Configuration file doesn't exist.")
        
        try:
            document = simplexml.SimpleXmlDocument(path)
            title = document.get_text_node_value("title", 0)
            webServiceAddress = document.get_text_node_value("predesys_webservice_address", 0)
            self.set_title(title)
            self.set_predesys_webservice_address(webServiceAddress)            
        except:
            raise coreexceptions.LoadFileException("Configuration load error.")
        
    def save_to_file(self, path):
        """Saves settings to a file.
        
        Keyword arguments:
        path -- File path to save settings (string)
        
        It raises an exception (ValueNotValidException) if path isn't a valid file path.
        It raises an exception (SaveFileException) if an error occurs saving the file.
        """
        if path == None or type(path) != str or path == "": raise coreexceptions.ValueNotValidException("Path must be a valid file path.")
        
        content = "<?xml version=\"1.0\" ?>\n"
        content += "<configuration>\n"
        content += "\t<title>%s</title>\n" % self.get_title()
        content += "\t<predesys_webservice_address>%s</predesys_webservice_address>\n" % self.get_predesys_webservice_address()
        content += "</configuration>"
        
        try:
            with open(path, "w") as configFile:
                configFile.write(content)
        except:
            raise coreexceptions.SaveFileException("Configuration save error.")
    
def load_configuration():
    """Loads configuration from CONFIG_FILE_1 file or CONFIG_FILE_2 file.
        
    If an error occurs, it sets the default configuration.
    """
    config = Configuration()
    
    if os.path.isfile(CONFIG_FILE):
        try:
            config.load_from_file(CONFIG_FILE)
        except:
            config.set_default_values()
    else:
        config.set_default_values()

    return config

def save_configuration(config):
    if config == None or not isinstance(config, Configuration): raise coreexceptions.ValueNotValidException("'config' must be a not null Configuration object.")
    
    if os.path.isfile(CONFIG_FILE):
        try:
            config.save_to_file(CONFIG_FILE)
        except:
            raise coreexceptions.SaveFileException("Saving configuration error")
    else:
        raise coreexceptions.SaveFileException("Saving configuration error")
